const Main          = imports.ui.main;
const St            = imports.gi.St;
const GObject       = imports.gi.GObject;
const Gio           = imports.gi.Gio;
const GLib          = imports.gi.GLib;
const PanelMenu     = imports.ui.panelMenu;
const PopupMenu     = imports.ui.popupMenu;

const LMAP_HOSTNAME = "lamp.lan"
const LMAP_PORT     = 2000
const TIMEOUT       = 1.0
const READ_RETRIES  = 5

let rlamp;

const RLamp = GObject.registerClass(
class RLamp extends PanelMenu.Button {
    // From https://github.com/jeffchannell/gnome-shell-socket/blob/master/client.js
    sendRequest(request)
    {
        try
        {
            let lampAddress = Gio.NetworkAddress.new(LMAP_HOSTNAME, LMAP_PORT);
            let client = new Gio.SocketClient();
            client.set_timeout(TIMEOUT);
            let connection = client.connect(lampAddress, null);
            if (!connection) {
                throw "Failed to connect to lamp on: " + lampAddress.get_hostname() 
            }
    
            let input = connection.get_input_stream();
            let output = connection.get_output_stream();
    
            output.write_bytes(new GLib.Bytes(''+request), null);
    
            let response = String.fromCharCode.apply(null, input.read_bytes(1024, null).get_data());
    
            connection.close(null);
            return response
        }
        catch(e)
        {
            return ""
        }
    }

    getStatus()
    {
        for(let i = 0; i < READ_RETRIES; i++)
        {
            let response = this.sendRequest("status");
            if(response.lenght < 1)
                continue;
            return (this.sendRequest("status") === "on");
        }
    }

    updateWidgets(shouldBeOn)
    {
        if (shouldBeOn)
        {
            this.lampSwitch.label.text = "Turn Lamp On";
            this.lampSwitch.setToggleState(true);
            this.rlampIcon.icon_name = 'weather-clear-day';
        }
        else
        {
            this.lampSwitch.label.text = "Turn Lamp Off";
            this.lampSwitch.setToggleState(false);
            this.rlampIcon.icon_name = 'weather-clear-night';
        }
    }
    
    commandSwitch(shouldBeOn) 
    {
        this.sendRequest(shouldBeOn ? "on" : "off");
        this.updateWidgets(shouldBeOn);
    }
    
    _init () 
    {
        super._init(0);
        
        // Panel icon
        this.rlampIcon = new St.Icon({
            icon_name : 'weather-clear-night',
            style_class : 'system-status-icon',
        });
        this.rlampIcon.connect('button-press-event', (item, state) => {
            this.updateWidgets(this.getStatus());
        });
        this.add_child(this.rlampIcon);

        // Lamp switch
        this.lampSwitch = new PopupMenu.PopupSwitchMenuItem('Lamp',
            false, {});
        this.lampSwitch.connect('toggled', (item, state) => {
            this.commandSwitch(state);
        });
        this.menu.addMenuItem(this.lampSwitch);
        
        // Get the initial state of the lamp and update the widgets
        this.updateWidgets(this.getStatus());
    }

});

function init() {
}

function enable() {
    rlamp = new RLamp();
    Main.panel.addToStatusArea('rlamp', rlamp, 1);
}

function disable() {
    rlamp.destroy();
}

